@@ -45,6 +45,7 @@ import java.io.FileNotFoundException;
 import java.io.IOException;
 
 import java.awt.Color;
+import java.awt.Font;
 import java.awt.Graphics2D;
 import java.awt.Insets;
 import java.awt.image.BufferedImage;
@@ -53,6 +54,7 @@ import java.awt.Image;
 import java.awt.Toolkit;
 import javax.imageio.ImageIO;
 import javax.imageio.stream.FileImageOutputStream;
+import java.lang.Character.UnicodeBlock;
 
 /**
  * Represents a logical mathematical formula that will be displayed (by creating a
@@ -78,8 +80,15 @@ import javax.imageio.stream.FileImageOutputStream;
  * this TeXFormula object!</b>
  */
 public class TeXFormula {
-    
-    public static final String VERSION = "0.9.4";
+
+    public static final String VERSION = "0.9.6";
+
+    public static final int SERIF = 0;
+    public static final int SANSSERIF = 1;
+    public static final int BOLD = 2;
+    public static final int ITALIC = 4;
+    public static final int ROMAN = 8;
+    public static final int TYPEWRITER = 16;
 
     // table for putting delimiters over and under formula's,
     // indexed by constants from "TeXConstants"
@@ -103,91 +112,105 @@ public class TeXFormula {
     // used as second index in "delimiterNames" table (over or under)
     private static final int OVER_DEL = 0;
     private static final int UNDER_DEL = 1;
-    
+
     // for comparing floats with 0
     protected static final float PREC = 0.0000001f;
-    
+
     // predefined TeXFormula's
-    public static Map<String,TeXFormula> predefinedTeXFormulas = new HashMap<String,TeXFormula>();
-    
+    public static Map<String, TeXFormula> predefinedTeXFormulas = new HashMap<String, TeXFormula>(150);
+    public static Map<String, String> predefinedTeXFormulasAsString = new HashMap<String, String>(150);
+
     // character-to-symbol and character-to-delimiter mappings
     public static String[] symbolMappings = new String[65536];
     public static String[] symbolTextMappings = new String[65536];
     public static String[] symbolFormulaMappings = new String[65536];
+    public static Map<Character.UnicodeBlock, FontInfos> externalFontMap = new HashMap<Character.UnicodeBlock, FontInfos>();
+
     public List<MiddleAtom> middle = new LinkedList();
-    
+
     private TeXParser parser;
-    
+
     static {
         // character-to-symbol and character-to-delimiter mappings
         TeXFormulaSettingsParser parser = new TeXFormulaSettingsParser();
-	parser.parseSymbolMappings(symbolMappings, symbolTextMappings);
-        
-        // predefined TeXFormula's
-	try {
-	    new PredefinedTeXFormulaParser("PredefinedCommands.xml", "Command").parse(MacroInfo.Commands);
-	    new PredefinedTeXFormulaParser("PredefinedTeXFormulas.xml", "TeXFormula").parse(predefinedTeXFormulas);
-	} catch (Exception e) {
-	    System.err.println(e.toString());
-	}
+        parser.parseSymbolMappings(symbolMappings, symbolTextMappings);
 
-	parser.parseSymbolToFormulaMappings(symbolFormulaMappings, symbolTextMappings);
-	
-	try {
-	    DefaultTeXFont.registerAlphabet((AlphabetRegistration) Class.forName("org.scilab.forge.jlatexmath.cyrillic.CyrillicRegistration").newInstance());
-	    DefaultTeXFont.registerAlphabet((AlphabetRegistration) Class.forName("org.scilab.forge.jlatexmath.greek.GreekRegistration").newInstance());
-	} catch (Exception e) { }
-	
-	//setDefaultDPI();
+        new PredefinedCommands();
+        new PredefinedTeXFormulas();
+        new PredefMacros();
+
+        parser.parseSymbolToFormulaMappings(symbolFormulaMappings, symbolTextMappings);
+
+        try {
+            DefaultTeXFont.registerAlphabet((AlphabetRegistration) Class.forName("org.scilab.forge.jlatexmath.cyrillic.CyrillicRegistration").newInstance());
+            DefaultTeXFont.registerAlphabet((AlphabetRegistration) Class.forName("org.scilab.forge.jlatexmath.greek.GreekRegistration").newInstance());
+        } catch (Exception e) { }
+
+        //setDefaultDPI();
     }
 
     public static void addSymbolMappings(String file) throws ResourceParseException {
-	FileInputStream in;
-	try {
-	    in = new FileInputStream(file);
-	} catch (FileNotFoundException e) {
-	    throw new ResourceParseException(file, e);
-	}
-	addSymbolMappings(in, file);
+        FileInputStream in;
+        try {
+            in = new FileInputStream(file);
+        } catch (FileNotFoundException e) {
+            throw new ResourceParseException(file, e);
+        }
+        addSymbolMappings(in, file);
     }
-    
+
     public static void addSymbolMappings(InputStream in, String name) throws ResourceParseException {
-	TeXFormulaSettingsParser tfsp = new TeXFormulaSettingsParser(in, name);
-	tfsp.parseSymbolMappings(symbolMappings, symbolTextMappings);
-	tfsp.parseSymbolToFormulaMappings(symbolFormulaMappings, symbolTextMappings);
+        TeXFormulaSettingsParser tfsp = new TeXFormulaSettingsParser(in, name);
+        tfsp.parseSymbolMappings(symbolMappings, symbolTextMappings);
+        tfsp.parseSymbolToFormulaMappings(symbolFormulaMappings, symbolTextMappings);
     }
-    
+
+    public static void registerExternalFont(Character.UnicodeBlock block, String sansserif, String serif) {
+        if (sansserif == null && serif == null) {
+            externalFontMap.remove(block);
+            return;
+        }
+        externalFontMap.put(block, new FontInfos(sansserif, serif));
+        if (block.equals(Character.UnicodeBlock.BASIC_LATIN)) {
+            predefinedTeXFormulas.clear();
+        }
+    }
+
+    public static void registerExternalFont(Character.UnicodeBlock block, String fontName) {
+        registerExternalFont(block, fontName, fontName);
+    }
+
     /**
      * Set the DPI of target
      * @param dpi the target DPI
      */
     public static void setDPITarget(float dpi) {
-	PIXELS_PER_POINT = dpi / 72f;
+        PIXELS_PER_POINT = dpi / 72f;
     }
 
     /**
      * Set the default target DPI to the screen dpi (only if we're in non-headless mode)
      */
     public static void setDefaultDPI() {
-	if (!GraphicsEnvironment.isHeadless()) {
-	    setDPITarget((float) Toolkit.getDefaultToolkit().getScreenResolution());
-	}
+        if (!GraphicsEnvironment.isHeadless()) {
+            setDPITarget((float) Toolkit.getDefaultToolkit().getScreenResolution());
+        }
     }
 
     // the root atom of the "atom tree" that represents the formula
     public Atom root = null;
-    
+
     // the current text style
     public String textStyle = null;
 
     public boolean isColored = false;
-    
+
     /**
      * Creates an empty TeXFormula.
      *
      */
     public TeXFormula() {
-	parser = new TeXParser("", this, false);
+        parser = new TeXParser("", this, false);
     }
 
     /**
@@ -201,27 +224,27 @@ public class TeXFormula {
     }
 
     public TeXFormula(String s, boolean firstpass) throws ParseException {
-	this.textStyle = null;
-	parser = new TeXParser(s, this, firstpass);
+        this.textStyle = null;
+        parser = new TeXParser(s, this, firstpass);
         parser.parse();
     }
-    
-   /*
-    * Creates a TeXFormula by parsing the given string in the given text style.
-    * Used when a text style command was found in the parse string.
-    */
+
+    /*
+     * Creates a TeXFormula by parsing the given string in the given text style.
+     * Used when a text style command was found in the parse string.
+     */
     public TeXFormula(String s, String textStyle) throws ParseException {
         this.textStyle = textStyle;
-	parser = new TeXParser(s, this);
+        parser = new TeXParser(s, this);
         parser.parse();
     }
 
     public TeXFormula(String s, String textStyle, boolean firstpass, boolean space) throws ParseException {
         this.textStyle = textStyle;
-	parser = new TeXParser(s, this, firstpass, space);
+        parser = new TeXParser(s, this, firstpass, space);
         parser.parse();
     }
-    
+
     /**
      * Creates a new TeXFormula that is a copy of the given TeXFormula.
      * <p>
@@ -230,17 +253,17 @@ public class TeXFormula {
      * @param f the formula to be copied
      */
     public TeXFormula(TeXFormula f) {
-        if (f != null)
+        if (f != null) {
             addImpl(f);
+        }
     }
 
-
     /**
      * Creates an empty TeXFormula.
      *
      */
     public TeXFormula(boolean isPartial) {
-	parser = new TeXParser(isPartial, "", this, false);
+        parser = new TeXParser(isPartial, "", this, false);
     }
 
     /**
@@ -254,43 +277,51 @@ public class TeXFormula {
     }
 
     public TeXFormula(boolean isPartial, String s, boolean firstpass) throws ParseException {
-	this.textStyle = null;
-	parser = new TeXParser(isPartial, s, this, firstpass);
-	if (isPartial) {
-	    try {
-		parser.parse();
-	    } catch (Exception e) { }
-	} else {
-	    parser.parse();
-	}
+        this.textStyle = null;
+        parser = new TeXParser(isPartial, s, this, firstpass);
+        if (isPartial) {
+            try {
+                parser.parse();
+            } catch (Exception e) { }
+        } else {
+            parser.parse();
+        }
     }
-    
-   /*
-    * Creates a TeXFormula by parsing the given string in the given text style.
-    * Used when a text style command was found in the parse string.
-    */
+
+    /*
+     * Creates a TeXFormula by parsing the given string in the given text style.
+     * Used when a text style command was found in the parse string.
+     */
     public TeXFormula(boolean isPartial, String s, String textStyle) throws ParseException {
         this.textStyle = textStyle;
-	parser = new TeXParser(isPartial, s, this);
-	if (isPartial) {
-	    try {
-		parser.parse();
-	    } catch (Exception e) { }
-	} else {
-	    parser.parse();
-	}
+        parser = new TeXParser(isPartial, s, this);
+        if (isPartial) {
+            try {
+                parser.parse();
+            } catch (Exception e) {
+                if (root == null) {
+                    root = new EmptyAtom();
+                }
+            }
+        } else {
+            parser.parse();
+        }
     }
 
     public TeXFormula(boolean isPartial, String s, String textStyle, boolean firstpass, boolean space) throws ParseException {
         this.textStyle = textStyle;
-	parser = new TeXParser(isPartial, s, this, firstpass, space);
-	if (isPartial) {
-	    try {
-		parser.parse();
-	    } catch (Exception e) { }
-	} else {
-	    parser.parse();
-	}
+        parser = new TeXParser(isPartial, s, this, firstpass, space);
+        if (isPartial) {
+            try {
+                parser.parse();
+            } catch (Exception e) {
+                if (root == null) {
+                    root = new EmptyAtom();
+                }
+            }
+        } else {
+            parser.parse();
+        }
     }
 
     /**
@@ -298,17 +329,21 @@ public class TeXFormula {
      * @return a partial TeXFormula containing the valid part of formula
      */
     public static TeXFormula getPartialTeXFormula(String formula) {
-	TeXFormula f = new TeXFormula();
-	if (formula == null) {
-	    f.add(new EmptyAtom());
-	    return f;
-	}
-	TeXParser parser = new TeXParser(true, formula, f);
-	try {
-	    parser.parse();
-	} catch (Exception e) { }
+        TeXFormula f = new TeXFormula();
+        if (formula == null) {
+            f.add(new EmptyAtom());
+            return f;
+        }
+        TeXParser parser = new TeXParser(true, formula, f);
+        try {
+            parser.parse();
+        } catch (Exception e) {
+            if (f.root == null) {
+                f.root = new EmptyAtom();
+            }
+        }
 
-	return f;
+        return f;
     }
 
     /**
@@ -316,16 +351,16 @@ public class TeXFormula {
      * with FOP.
      */
     public static void registerFonts(boolean b) {
-	DefaultTeXFontParser.registerFonts(b);
+        DefaultTeXFontParser.registerFonts(b);
     }
-    
+
     /**
      * Change the text of the TeXFormula and regenerate the root
      *
      * @param ltx the latex formula
-     */ 
+     */
     public void setLaTeX(String ltx) throws ParseException {
-	parser.reset(ltx);
+        parser.reset(ltx);
         if (ltx != null && ltx.length() != 0)
             parser.parse();
     }
@@ -335,8 +370,8 @@ public class TeXFormula {
      */
     public TeXFormula add(Atom el) {
         if (el != null) {
-	    if (el instanceof MiddleAtom)
-		middle.add((MiddleAtom)el);
+            if (el instanceof MiddleAtom)
+                middle.add((MiddleAtom)el);
             if (root == null)
                 root = el;
             else {
@@ -365,19 +400,19 @@ public class TeXFormula {
         }
         return this;
     }
-    
+
     public TeXFormula append(String s) throws ParseException {
-	return append(false, s);
+        return append(false, s);
     }
 
     public TeXFormula append(boolean isPartial, String s) throws ParseException {
-	if (s != null && s.length() != 0) {
+        if (s != null && s.length() != 0) {
             TeXParser tp = new TeXParser(isPartial, s, this);
-	    tp.parse();
+            tp.parse();
         }
         return this;
     }
-    
+
     /**
      * Inserts the given TeXFormula at the end of the current TeXFormula.
      *
@@ -388,7 +423,7 @@ public class TeXFormula {
         addImpl (f);
         return this;
     }
-    
+
     private void addImpl(TeXFormula f) {
         if (f.root != null) {
             // special copy-treatment for Mrow as a root!!
@@ -398,18 +433,18 @@ public class TeXFormula {
                 add(f.root);
         }
     }
-    
+
     public void setLookAtLastAtom(boolean b) {
-	if (root instanceof RowAtom)
-	    ((RowAtom)root).lookAtLastAtom = b;
+        if (root instanceof RowAtom)
+            ((RowAtom)root).lookAtLastAtom = b;
     }
 
     public boolean getLookAtLastAtom() {
-	if (root instanceof RowAtom)
-	    return ((RowAtom)root).lookAtLastAtom;
-	return false;
+        if (root instanceof RowAtom)
+            return ((RowAtom)root).lookAtLastAtom;
+        return false;
     }
-    
+
     /**
      * Centers the current TeXformula vertically on the axis (defined by the parameter
      * "axisheight" in the resource "DefaultTeXFont.xml".
@@ -420,15 +455,15 @@ public class TeXFormula {
         root = new VCenteredAtom(root);
         return this;
     }
-    
+
     public static void addPredefinedTeXFormula(InputStream xmlFile) throws ResourceParseException {
-	new PredefinedTeXFormulaParser(xmlFile, "TeXFormula").parse(predefinedTeXFormulas);
+        new PredefinedTeXFormulaParser(xmlFile, "TeXFormula").parse(predefinedTeXFormulas);
     }
-    
+
     public static void addPredefinedCommands(InputStream xmlFile) throws ResourceParseException {
-	new PredefinedTeXFormulaParser(xmlFile, "Command").parse(MacroInfo.Commands);
+        new PredefinedTeXFormulaParser(xmlFile, "Command").parse(MacroInfo.Commands);
     }
-    
+
     /**
      * Inserts a strut box (whitespace) with the given width, height and depth (in
      * the given unit) at the end of the current TeXFormula.
@@ -439,13 +474,13 @@ public class TeXFormula {
      * @param depth the depth of the strut box
      * @return the modified TeXFormula
      * @throws InvalidUnitException if the given integer value does not represent
-     * 			a valid unit
+     *                  a valid unit
      */
     public TeXFormula addStrut(int unit, float width, float height, float depth)
-    throws InvalidUnitException {
+        throws InvalidUnitException {
         return add(new SpaceAtom(unit, width, height, depth));
     }
-    
+
     /**
      * Inserts a strut box (whitespace) with the given width, height and depth (in
      * the given unit) at the end of the current TeXFormula.
@@ -453,13 +488,13 @@ public class TeXFormula {
      * @param type thinmuskip, medmuskip or thickmuskip (from {@link TeXConstants})
      * @return the modified TeXFormula
      * @throws InvalidUnitException if the given integer value does not represent
-     * 			a valid unit
+     *                  a valid unit
      */
     public TeXFormula addStrut(int type)
-    throws InvalidUnitException {
+        throws InvalidUnitException {
         return add(new SpaceAtom(type));
     }
-    
+
     /**
      * Inserts a strut box (whitespace) with the given width (in widthUnits), height
      * (in heightUnits) and depth (in depthUnits) at the end of the current TeXFormula.
@@ -472,24 +507,48 @@ public class TeXFormula {
      * @param depth the depth of the strut box
      * @return the modified TeXFormula
      * @throws InvalidUnitException if the given integer value does not represent
-     * 			a valid unit
+     *                  a valid unit
      */
     public TeXFormula addStrut(int widthUnit, float width, int heightUnit,
-            float height, int depthUnit, float depth) throws InvalidUnitException {
+                               float height, int depthUnit, float depth) throws InvalidUnitException {
         return add(new SpaceAtom(widthUnit, width, heightUnit, height, depthUnit,
-                depth));
+                                 depth));
     }
-    
-   /*
-    * Convert this TeXFormula into a box, starting form the given style
-    */
+
+    /*
+     * Convert this TeXFormula into a box, starting form the given style
+     */
     private Box createBox(TeXEnvironment style) {
         if (root == null)
             return new StrutBox(0, 0, 0, 0);
         else
             return root.createBox(style);
     }
-        
+
+    private DefaultTeXFont createFont(float size, int type) {
+        DefaultTeXFont dtf = new DefaultTeXFont(size);
+        if (type == 0) {
+            dtf.setSs(false);
+        }
+        if ((type & ROMAN) != 0) {
+            dtf.setRoman(true);
+        }
+        if ((type & TYPEWRITER) != 0) {
+            dtf.setTt(true);
+        }
+        if ((type & SANSSERIF) != 0) {
+            dtf.setSs(true);
+        }
+        if ((type & ITALIC) != 0) {
+            dtf.setIt(true);
+        }
+        if ((type & BOLD) != 0) {
+            dtf.setBold(true);
+        }
+
+        return dtf;
+    }
+
     /**
      * Creates a TeXIcon from this TeXFormula using the default TeXFont in the given
      * point size and starting from the given TeX style. If the given integer value
@@ -501,27 +560,50 @@ public class TeXFormula {
      * @return the created TeXIcon
      */
     public TeXIcon createTeXIcon(int style, float size) {
-	TeXEnvironment te = new TeXEnvironment(style, new DefaultTeXFont(size));
-	Box box = createBox(te);
-	TeXIcon ti = new TeXIcon(box, size);
-	ti.isColored = te.isColored;
+        TeXEnvironment te = new TeXEnvironment(style, new DefaultTeXFont(size));
+        Box box = createBox(te);
+        TeXIcon ti = new TeXIcon(box, size);
+        ti.isColored = te.isColored;
+        return ti;
+    }
+
+    public TeXIcon createTeXIcon(int style, float size, int type) {
+        TeXEnvironment te = new TeXEnvironment(style, createFont(size, type));
+        Box box = createBox(te);
+        TeXIcon ti = new TeXIcon(box, size);
+        ti.isColored = te.isColored;
+        return ti;
+    }
+
+    public TeXIcon createTeXIcon(int style, float size, int type, Color fgcolor) {
+        TeXEnvironment te = new TeXEnvironment(style, createFont(size, type));
+        Box box = createBox(te);
+        TeXIcon ti = new TeXIcon(box, size);
+        if (fgcolor != null) {
+            ti.setForeground(fgcolor);
+        }
+        ti.isColored = te.isColored;
         return ti;
     }
 
     public TeXIcon createTeXIcon(int style, float size, boolean trueValues) {
-	TeXEnvironment te = new TeXEnvironment(style, new DefaultTeXFont(size));
-	Box box = createBox(te);
-	TeXIcon ti = new TeXIcon(box, size, trueValues);
-	ti.isColored = te.isColored;
+        TeXEnvironment te = new TeXEnvironment(style, new DefaultTeXFont(size));
+        Box box = createBox(te);
+        TeXIcon ti = new TeXIcon(box, size, trueValues);
+        ti.isColored = te.isColored;
         return ti;
     }
 
     public TeXIcon createTeXIcon(int style, float size, int widthUnit, float textwidth, int align) {
-	TeXEnvironment te = new TeXEnvironment(style, new DefaultTeXFont(size), widthUnit, textwidth);
-	Box box = createBox(te);
-	HorizontalBox hb = new HorizontalBox(box, te.getTextwidth(), align);
-	TeXIcon ti = new TeXIcon(hb, size, true);
-	ti.isColored = te.isColored;
+        return createTeXIcon(style, size, 0, widthUnit, textwidth, align);
+    }
+
+    public TeXIcon createTeXIcon(int style, float size, int type, int widthUnit, float textwidth, int align) {
+        TeXEnvironment te = new TeXEnvironment(style, createFont(size, type), widthUnit, textwidth);
+        Box box = createBox(te);
+        HorizontalBox hb = new HorizontalBox(box, te.getTextwidth(), align);
+        TeXIcon ti = new TeXIcon(hb, size, true);
+        ti.isColored = te.isColored;
         return ti;
     }
 
@@ -534,13 +616,13 @@ public class TeXFormula {
         Graphics2D g2 = image.createGraphics();
         if (bg != null && !transparency) {
             g2.setColor(bg);
-	    g2.fillRect(0, 0, w, h);
+            g2.fillRect(0, 0, w, h);
         }
 
         icon.setForeground(fg);
         icon.paintIcon(null, g2, 0, 0);
         try {
-            FileImageOutputStream imout = new FileImageOutputStream(new File(out)); 
+            FileImageOutputStream imout = new FileImageOutputStream(new File(out));
             ImageIO.write(image, format, imout);
             imout.flush();
             imout.close();
@@ -548,9 +630,9 @@ public class TeXFormula {
             System.err.println("I/O error : Cannot generate " + out);
         }
 
-	g2.dispose();
+        g2.dispose();
     }
-    
+
     public void createPNG(int style, float size, String out, Color bg, Color fg) {
         createImage("png", style, size, out, bg, fg, bg == null);
     }
@@ -560,7 +642,7 @@ public class TeXFormula {
     }
 
     public void createJPEG(int style, float size, String out, Color bg, Color fg) {
-        //There is a bug when a BufferedImage has a component alpha so we disabel it 
+        //There is a bug when a BufferedImage has a component alpha so we disabel it
         createImage("jpeg", style, size, out, bg, fg, false);
     }
 
@@ -570,9 +652,9 @@ public class TeXFormula {
      * @param size the size
      * @param transparency, if true the background is transparent
      * @return the generated image
-     */ 
+     */
     public static Image createBufferedImage(String formula, int style, float size, Color fg, Color bg) throws ParseException {
-	TeXFormula f = new TeXFormula(formula);
+        TeXFormula f = new TeXFormula(formula);
         TeXIcon icon = f.createTeXIcon(style, size);
         icon.setInsets(new Insets(2, 2, 2, 2));
         int w = icon.getIconWidth(), h = icon.getIconHeight();
@@ -581,14 +663,14 @@ public class TeXFormula {
         Graphics2D g2 = image.createGraphics();
         if (bg != null) {
             g2.setColor(bg);
-	    g2.fillRect(0, 0, w, h);
+            g2.fillRect(0, 0, w, h);
         }
 
-	icon.setForeground(fg == null ? Color.BLACK : fg);
+        icon.setForeground(fg == null ? Color.BLACK : fg);
         icon.paintIcon(null, g2, 0, 0);
-	g2.dispose();
-	
-	return image;
+        g2.dispose();
+
+        return image;
     }
 
     /**
@@ -599,7 +681,7 @@ public class TeXFormula {
      * @return the generated image
      */ static int toto=0;
     public Image createBufferedImage(int style, float size, Color fg, Color bg) throws ParseException {
-	TeXIcon icon = createTeXIcon(style, size);
+        TeXIcon icon = createTeXIcon(style, size);
         icon.setInsets(new Insets(2, 2, 2, 2));
         int w = icon.getIconWidth(), h = icon.getIconHeight();
 
@@ -607,18 +689,18 @@ public class TeXFormula {
         Graphics2D g2 = image.createGraphics();
         if (bg != null) {
             g2.setColor(bg);
-	    g2.fillRect(0, 0, w, h);
+            g2.fillRect(0, 0, w, h);
         }
 
-	icon.setForeground(fg == null ? Color.BLACK : fg);
+        icon.setForeground(fg == null ? Color.BLACK : fg);
         icon.paintIcon(null, g2, 0, 0);
-	g2.dispose();
+        g2.dispose();
 
-	return image;
+        return image;
     }
 
     public void setDEBUG(boolean b) {
-	Box.DEBUG = b;
+        Box.DEBUG = b;
     }
 
     /**
@@ -641,7 +723,7 @@ public class TeXFormula {
         }
         return this;
     }
-    
+
     /**
      * Changes the (foreground) color of the <i>current</i> TeXFormula into the given color.
      * By default, the foreground color of a TeXFormula is the foreground color of the
@@ -663,7 +745,7 @@ public class TeXFormula {
         }
         return this;
     }
-    
+
     /**
      * Sets a fixed left and right type of the current TeXFormula. This has an influence
      * on the glue that will be inserted before and after this TeXFormula.
@@ -672,10 +754,10 @@ public class TeXFormula {
      * @param rightType atom type constant (from TeXConstants)
      * @return the modified TeXFormula
      * @throws InvalidAtomTypeException if the given integer value does not represent
-     * 			a valid atom type
+     *                  a valid atom type
      */
     public TeXFormula setFixedTypes(int leftType, int rightType)
-    throws InvalidAtomTypeException {
+        throws InvalidAtomTypeException {
         root = new TypedAtom(leftType, rightType, root);
         return this;
     }
@@ -686,13 +768,31 @@ public class TeXFormula {
      * @param name the name of the predefined TeXFormula
      * @return a copy of the predefined TeXFormula
      * @throws FormulaNotFoundException if no predefined TeXFormula is found with the
-     * 			given name
+     *                  given name
      */
     public static TeXFormula get(String name) throws FormulaNotFoundException {
         TeXFormula formula = predefinedTeXFormulas.get(name);
-        if (formula == null)
-            throw new FormulaNotFoundException(name);
-        else
+        if (formula == null) {
+            String f = predefinedTeXFormulasAsString.get(name);
+            if (f == null) {
+                throw new FormulaNotFoundException(name);
+            }
+            TeXFormula tf = new TeXFormula(f);
+            predefinedTeXFormulas.put(name, tf);
+            return tf;
+        } else {
             return new TeXFormula(formula);
+        }
+    }
+
+    static class FontInfos {
+
+        String sansserif;
+        String serif;
+
+        FontInfos(String sansserif, String serif) {
+            this.sansserif = sansserif;
+            this.serif = serif;
+        }
     }
 }
