@@ -29,7 +29,10 @@
 package org.scilab.forge.jlatexmath;
 
 import java.awt.Color;
+import java.awt.Font;
 import java.lang.Character.UnicodeBlock;
+import java.util.HashSet;
+import java.util.Set;
 
 /**
  * This class implements a parser for LaTeX' formulas.
@@ -108,6 +111,16 @@ public class TeXParser {
 
     protected static boolean isLoading = false;
 
+    private static final Set<String> unparsedContents = new HashSet<String>(6);
+    static {
+        unparsedContents.add("jlmDynamic");
+        unparsedContents.add("jlmText");
+        unparsedContents.add("jlmTextit");
+        unparsedContents.add("jlmTextbf");
+        unparsedContents.add("jlmTextitbf");
+        unparsedContents.add("jlmExternalFont");
+    }
+
     /**
      * Create a new TeXParser
      *
@@ -406,8 +419,7 @@ public class TeXParser {
                         atIsLetter++;
                     else if ("makeatother".equals(com))
                         atIsLetter--;
-                    else if ("jlmDynamic".equals(com)) {
-                        //the argument of jlmDynamic is skipped
+                    else if (unparsedContents.contains(com)) {
                         getOptsArgs(1, 0);
                     }
                     break;
@@ -628,7 +640,20 @@ public class TeXParser {
                 case DOLLAR :
                     pos++;
                     if (!ignoreWhiteSpace) {// We are in a mbox
-                        formula.add(new MathAtom(new TeXFormula(isPartial, getDollarGroup(DOLLAR), false).root));
+                        int style = TeXConstants.STYLE_TEXT;
+                        boolean doubleDollar = false;
+                        if (parseString.charAt(pos) == DOLLAR) {
+                            style = TeXConstants.STYLE_DISPLAY;
+                            doubleDollar = true;
+                            pos++;
+                        }
+
+                        formula.add(new MathAtom(new TeXFormula(isPartial, getDollarGroup(DOLLAR), false).root, style));
+                        if (doubleDollar) {
+                            if (parseString.charAt(pos) == DOLLAR) {
+                                pos++;
+                            }
+                        }
                     }
                     break;
                 case ESCAPE :
@@ -668,7 +693,7 @@ public class TeXParser {
                     if (ignoreWhiteSpace) {
                         formula.add(new CumulativeScriptsAtom(getLastAtom(), null, SymbolAtom.get("prime")));
                     } else {
-                        formula.add(convertCharacter(PRIME));
+                        formula.add(convertCharacter(PRIME, true));
                     }
                     pos++;
                     break;
@@ -676,12 +701,12 @@ public class TeXParser {
                     if (ignoreWhiteSpace) {
                         formula.add(new CumulativeScriptsAtom(getLastAtom(), null, SymbolAtom.get("backprime")));
                     } else {
-                        formula.add(convertCharacter(BACKPRIME));
+                        formula.add(convertCharacter(BACKPRIME, true));
                     }
                     pos++;
                     break;
                 default :
-                    formula.add(convertCharacter(ch));
+                    formula.add(convertCharacter(ch, false));
                     pos++;
                 }
             }
@@ -800,8 +825,9 @@ public class TeXParser {
             }
 
             return parseString.substring(spos + 1, pos - 1);
-        } else
+        } else {
             throw new ParseException("missing '" + open + "'!");
+        }
     }
 
     /** Get the contents between two strings as in \begin{foo}...\end{foo}
@@ -880,6 +906,9 @@ public class TeXParser {
         }
 
         if (group != 0) {
+            if (isPartial) {
+                return buf.toString();
+            }
             throw new ParseException("The token " + open + " must be closed by " + close);
         }
 
@@ -922,9 +951,10 @@ public class TeXParser {
             }
             return at;
         }
-        pos++;
 
-        return convertCharacter(ch);
+        Atom at = convertCharacter(ch, true);
+	pos++;
+	return at;
     }
 
     public String getOverArgument() throws ParseException {
@@ -990,7 +1020,7 @@ public class TeXParser {
      * @return the corresponding atom
      * @throws ParseException if the character is unknown
      */
-    public Atom convertCharacter(char c) throws ParseException {
+    public Atom convertCharacter(char c, boolean oneChar) throws ParseException {
         if (ignoreWhiteSpace) {// The Unicode Greek letters in math mode are not drawn with the Greek font
             if (c >= 945 && c <= 969) {
                 return SymbolAtom.get(TeXFormula.symbolMappings[c]);
@@ -1008,6 +1038,24 @@ public class TeXParser {
 
             String symbolName = TeXFormula.symbolMappings[c];
             if (symbolName == null && (TeXFormula.symbolFormulaMappings == null || TeXFormula.symbolFormulaMappings[c] == null)) {
+                TeXFormula.FontInfos fontInfos = TeXFormula.externalFontMap.get(block);
+		if (fontInfos != null) {
+		    if (oneChar) {
+			return new JavaFontRenderingAtom(Character.toString(c), fontInfos);
+		    }
+                    int start = pos++;
+                    int end = len - 1;
+                    while (pos < len) {
+                        c = parseString.charAt(pos);
+                        if (!Character.UnicodeBlock.of(c).equals(block)) {
+                            end = --pos;
+                            break;
+                        }
+                        pos++;
+                    }
+                    return new JavaFontRenderingAtom(parseString.substring(start, end + 1), fontInfos);
+                }
+
                 if (!isPartial) {
                     throw new ParseException("Unknown character : '"
                                              + Character.toString(c) + "' (or " + ((int) c) + ")");
@@ -1033,9 +1081,27 @@ public class TeXParser {
                                              + symbolName + "'!", e);
                 }
             }
-        } else
+        } else {
             // alphanumeric character
+	    TeXFormula.FontInfos fontInfos = TeXFormula.externalFontMap.get(Character.UnicodeBlock.BASIC_LATIN);
+	    if (fontInfos != null) {
+		if (oneChar) {
+		    return new JavaFontRenderingAtom(Character.toString(c), fontInfos);
+		}
+                int start = pos++;
+                int end = len - 1;
+                while (pos < len) {
+                    c = parseString.charAt(pos);
+                    if (((c < '0' || c > '9') && (c < 'a' || c > 'z') && (c < 'A' || c > 'Z'))) {
+                        end = --pos;
+                        break;
+                    }
+                    pos++;
+                }
+                return new JavaFontRenderingAtom(parseString.substring(start, end + 1), fontInfos);
+            }
             return new CharAtom(c, formula.textStyle);
+        }
     }
 
     private String getCommand() {
